/*
 * QEstEidCommon
 *
 * Copyright (C) 2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "MacMenuBar.h"

#include <QtCore/QEvent>
#include <QtGui/QApplication>

void qt_mac_set_dock_menu( QMenu *menu );

MacMenuBar::MacMenuBar( bool enableDock )
	: QMenuBar()
	, dock(0)
	, dockSeparator(0)
	, windowGroup(0)
{
	file = addMenu( tr("&File") );
	if( enableDock )
	{
		dock = new QMenu;
		windowGroup = new QActionGroup( dock );
		qApp->installEventFilter( this );
		connect( dock, SIGNAL(triggered(QAction*)), SLOT(activateWindow(QAction*)) );
		qt_mac_set_dock_menu( dock );
	}
}

MacMenuBar::~MacMenuBar()
{
	//delete dock;
}

void MacMenuBar::activateWindow( QAction *a )
{
	if( QWidget *w = a->data().value<QWidget*>() )
	{
		w->activateWindow();
		w->showNormal();
		w->raise();
	}
}

QAction* MacMenuBar::addAction( ActionType type, const QObject *receiver, const char *member )
{
	QAction *a = file->addAction( typeName( type ), receiver, member );
	switch( type )
	{
	case AboutAction: a->setMenuRole( QAction::AboutRole ); break;
	case CloseAction: a->setShortcut( Qt::CTRL + Qt::Key_W ); break;
	case PreferencesAction: a->setMenuRole( QAction::PreferencesRole ); break;
	default: break;
	}
	actions[type] = a;
	return a;
}

QMenu* MacMenuBar::dockMenu() const { return dock; }

bool MacMenuBar::eventFilter( QObject *o, QEvent *e )
{
	if( dock )
	{
		switch( e->type() )
		{
		case QEvent::Close:
		case QEvent::Destroy:
			Q_FOREACH( QAction *a, windowGroup->actions() )
				if( o == a->data().value<QWidget*>() )
					delete a;
			if( windowGroup->actions().isEmpty() && dockSeparator )
			{
				dockSeparator->deleteLater();
				dockSeparator = 0;
			}
			return true;
		case QEvent::WindowActivate:
			Q_FOREACH( QAction *a, windowGroup->actions() )
				a->setChecked( o == a->data().value<QWidget*>() );
			if( !windowGroup->checkedAction() )
			{
				QAction *a = new QAction( o->property( "windowTitle" ).toString(), dock );
				a->setCheckable( true );
				a->setData( QVariant::fromValue( qobject_cast<QWidget*>(o) ) );
				a->setActionGroup( windowGroup );

				if( !dockSeparator )
					dockSeparator = dock->insertSeparator( dock->actions().value(0) );
				dock->insertAction( dockSeparator, a );
			}
			return true;
		case QEvent::WindowTitleChange:
			Q_FOREACH( QAction *a, windowGroup->actions() )
				if( o == a->data().value<QWidget*>() )
					a->setText( o->property( "windowTitle" ).toString() );
			return true;
		default: break;
		}
	}
	switch( e->type() )
	{
	case QEvent::LanguageChange:
		file->setTitle( tr("&File") );
		for( QHash<ActionType,QAction*>::const_iterator i = actions.constBegin();
			i != actions.constEnd(); ++i )
			i.value()->setText( typeName( i.key() ) );
		break;
	default: break;
	}
	return QMenuBar::eventFilter( o, e );
}

QMenu* MacMenuBar::fileMenu() const { return file; }

QString MacMenuBar::typeName( ActionType type ) const
{
	switch( type )
	{
	case AboutAction: return tr("Info");
	case CloseAction: return tr("Close");
	case PreferencesAction: return tr("Settings");
	default: return QString();
	}
}
