/*
 * QEstEidUtil
 *
 * Copyright (C) 2009-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2009-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "SettingsDialog.h"

#include <common/Common.h>

#include <QtGui/QMessageBox>
#include <QtGui/QPushButton>

SettingsDialog::SettingsDialog( QWidget *parent )
:	QDialog( parent )
{
	setupUi( this );
	setWindowModality( Qt::WindowModal );
	setAttribute( Qt::WA_DeleteOnClose, true );
	updateInterval->addItem( tr("Once a day"), "-daily" );
	updateInterval->addItem( tr("Once a week"), "-weekly" );
	updateInterval->addItem( tr("Once a month"), "-monthly" );
	updateInterval->addItem( tr("Never"), "-never" );
	updateInterval->addItem( tr("Remove"), "-remove" );
	update = buttonBox->addButton( tr("Check for updates"), QDialogButtonBox::ActionRole );
#ifdef Q_OS_MAC
	clear = buttonBox->addButton( tr("Clear token cache"), QDialogButtonBox::ActionRole );
#endif
}

void SettingsDialog::buttonClicked( QAbstractButton *button )
{
	QPushButton *b = qobject_cast<QPushButton*>( button );
	if( b == buttonBox->button( QDialogButtonBox::Close ) )
		done( 0 );
	else if( b == buttonBox->button( QDialogButtonBox::Save ) )
	{
#ifdef Q_OS_MAC
		Common::runPrivileged( qApp->applicationDirPath() + "/id-updater.app/Contents/MacOS/id-updater",
#else
		Common::runPrivileged( "id-updater",
#endif
			QStringList() << updateInterval->itemData( updateInterval->currentIndex() ).toString() );
		done( 1 );
	}
	else if( b == update )
	{
		Common::startDetached( "id-updater" );
		done( 1 );
	}
	else if( b == clear )
	{
		QMessageBox::warning( this, tr("Certificate update"),
			tr("Remove card from reader to clean TokenCache. It requires user password") );
		if( Common::runPrivileged( "/bin/sh",
				QStringList() << "-c" << "/bin/rm -rf /var/db/TokenCache/tokens/com.apple.tokend.opensc*" ) )
			QMessageBox::information( this, tr("Certificate update"), tr("TokenCache succesfully cleaned") );
		else
			QMessageBox::warning( this, tr("Certificate update"), tr("TokenCache cleanup failed") );
		done( 1 );
	}
}
